@@ -4,11 +4,13 @@ import copy
 from ctypes import CDLL, POINTER, c_double, c_char_p, pointer
 from ctypes.util import find_library
 import fnmatch
+from collections import MutableMapping
 
 import redis
 import redis.client
 
 
+__version__ = '0.3.1'
 DATABASES = {}
 
 _libc = CDLL(find_library('c'))
@@ -17,10 +19,30 @@ _libc.strtod.argtypes = [c_char_p, POINTER(c_char_p)]
 _strtod = _libc.strtod
 
 
+class _StrKeyDict(MutableMapping):
+    def __init__(self, *args, **kwargs):
+        self._dict = dict(*args, **kwargs)
+
+    def __getitem__(self, key):
+        return self._dict[str(key)]
+
+    def __setitem__(self, key, value):
+        self._dict[str(key)] = value
+
+    def __delitem__(self, key):
+        del self._dict[key]
+
+    def __len__(self):
+        return len(self._dict)
+
+    def __iter__(self):
+        return iter(self._dict)
+
+
 class FakeStrictRedis(object):
     def __init__(self, db=0, **kwargs):
         if db not in DATABASES:
-            DATABASES[db] = {}
+            DATABASES[db] = _StrKeyDict()
         self._db = DATABASES[db]
         self._db_num = db
 
@@ -130,6 +152,9 @@ class FakeStrictRedis(object):
     def persist(self, name):
         pass
 
+    def ping(self):
+        return True
+
     def randomkey(self):
         pass
 
@@ -410,11 +435,12 @@ class FakeStrictRedis(object):
         self._db.get(name, []).insert(index, value)
 
     def rpoplpush(self, src, dst):
-        el = self._db.get(src).pop()
-        try:
-            self._db[dst].insert(0, el)
-        except KeyError:
-            self._db[dst] = [el]
+        el = self.rpop(src)
+        if el is not None:
+            try:
+                self._db[dst].insert(0, el)
+            except KeyError:
+                self._db[dst] = [el]
         return el
 
     def blpop(self, keys, timeout=0):
@@ -442,11 +468,12 @@ class FakeStrictRedis(object):
                 return (key, self._db[key].pop())
 
     def brpoplpush(self, src, dst, timeout=0):
-        el = self._db.get(src).pop()
-        try:
-            self._db[dst].insert(0, el)
-        except KeyError:
-            self._db[dst] = [el]
+        el = self.rpop(src)
+        if el is not None:
+            try:
+                self._db[dst].insert(0, el)
+            except KeyError:
+                self._db[dst] = [el]
         return el
 
     def hdel(self, name, *keys):
@@ -539,7 +566,7 @@ class FakeStrictRedis(object):
     def sdiff(self, keys, *args):
         "Return the difference of sets specified by ``keys``"
         all_keys = redis.client.list_or_args(keys, args)
-        diff = self._db.get(all_keys[0], set())
+        diff = self._db.get(all_keys[0], set()).copy()
         for key in all_keys[1:]:
             diff -= self._db.get(key, set())
         return diff
@@ -877,6 +904,9 @@ class FakeStrictRedis(object):
             keys_weights = [(k, 1) for k in keys]
         for key, weight in keys_weights:
             current_zset = self._db.get(key, {})
+            if isinstance(current_zset, set):
+                # When casting set to zset redis uses a default score of 1.0
+                current_zset = dict((k, 1.0) for k in current_zset)
             for el in current_zset:
                 if not should_include(el):
                     continue
